from asdl import pybase
from pylib.collections_ import OrderedDict
from typing import Optional, List, Tuple, Dict, Any, cast, TYPE_CHECKING

if TYPE_CHECKING:
  from _devbuild.gen.syntax_asdl import ArgList, command_t, re_t, redir_loc_t, Token, proc_sig_t

from _devbuild.gen.id_kind_asdl import Id_t
from _devbuild.gen.id_kind_asdl import Id_str


from asdl import runtime  # For runtime.NO_SPID
from asdl.runtime import NewRecord, NewLeaf
from _devbuild.gen.hnode_asdl import color_e, hnode, hnode_e, hnode_t, field

class cmd_value_e(object):
  Argv = 1
  Assign = 2

_cmd_value_str = {
  1: 'cmd_value.Argv',
  2: 'cmd_value.Assign',
}

def cmd_value_str(tag):
  # type: (int) -> str
  return _cmd_value_str[tag]

class cmd_value_t(pybase.CompoundObj):
  def tag_(self):
    # type: () -> int
    return self._type_tag
  pass

class cmd_value__Argv(cmd_value_t):
  _type_tag = 1
  __slots__ = ('argv', 'arg_spids', 'typed_args')

  def __init__(self, argv, arg_spids, typed_args):
    # type: (Optional[List[str]], Optional[List[int]], Optional[Optional[ArgList]]) -> None
    self.argv = argv
    self.arg_spids = arg_spids
    self.typed_args = typed_args

  @staticmethod
  def Create():
    # type: () -> cmd_value__Argv
    return cmd_value__Argv([], [], None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('cmd_value.Argv')
    L = out_node.fields

    if self.argv is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.argv:
        x0.children.append(NewLeaf(i0, color_e.StringConst))
      L.append(field('argv', x0))

    if self.arg_spids is not None:  # array
      x1 = hnode.Array([])
      for i1 in self.arg_spids:
        x1.children.append(hnode.Leaf(str(i1), color_e.OtherConst))
      L.append(field('arg_spids', x1))

    if self.typed_args is not None:  # maybe
      x2 = self.typed_args.PrettyTree()
      L.append(field('typed_args', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('cmd_value.Argv')
    L = out_node.fields
    if self.argv is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.argv:
        x0.children.append(NewLeaf(i0, color_e.StringConst))
      L.append(field('argv', x0))

    if self.arg_spids is not None:  # array
      x1 = hnode.Array([])
      for i1 in self.arg_spids:
        x1.children.append(hnode.Leaf(str(i1), color_e.OtherConst))
      L.append(field('arg_spids', x1))

    if self.typed_args is not None:  # maybe
      x2 = self.typed_args.AbbreviatedTree()
      L.append(field('typed_args', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class cmd_value__Assign(cmd_value_t):
  _type_tag = 2
  __slots__ = ('builtin_id', 'argv', 'arg_spids', 'pairs')

  def __init__(self, builtin_id, argv, arg_spids, pairs):
    # type: (int, Optional[List[str]], Optional[List[int]], Optional[List[assign_arg]]) -> None
    self.builtin_id = builtin_id
    self.argv = argv
    self.arg_spids = arg_spids
    self.pairs = pairs

  @staticmethod
  def Create():
    # type: () -> cmd_value__Assign
    return cmd_value__Assign(-1, [], [], [])

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('cmd_value.Assign')
    L = out_node.fields

    x0 = hnode.Leaf(str(self.builtin_id), color_e.OtherConst)
    L.append(field('builtin_id', x0))

    if self.argv is not None:  # array
      x1 = hnode.Array([])
      for i1 in self.argv:
        x1.children.append(NewLeaf(i1, color_e.StringConst))
      L.append(field('argv', x1))

    if self.arg_spids is not None:  # array
      x2 = hnode.Array([])
      for i2 in self.arg_spids:
        x2.children.append(hnode.Leaf(str(i2), color_e.OtherConst))
      L.append(field('arg_spids', x2))

    if self.pairs is not None:  # array
      x3 = hnode.Array([])
      for i3 in self.pairs:
        x3.children.append(i3.PrettyTree())
      L.append(field('pairs', x3))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('cmd_value.Assign')
    L = out_node.fields
    x0 = hnode.Leaf(str(self.builtin_id), color_e.OtherConst)
    L.append(field('builtin_id', x0))

    if self.argv is not None:  # array
      x1 = hnode.Array([])
      for i1 in self.argv:
        x1.children.append(NewLeaf(i1, color_e.StringConst))
      L.append(field('argv', x1))

    if self.arg_spids is not None:  # array
      x2 = hnode.Array([])
      for i2 in self.arg_spids:
        x2.children.append(hnode.Leaf(str(i2), color_e.OtherConst))
      L.append(field('arg_spids', x2))

    if self.pairs is not None:  # array
      x3 = hnode.Array([])
      for i3 in self.pairs:
        x3.children.append(i3.AbbreviatedTree())
      L.append(field('pairs', x3))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class cmd_value(object):
  Argv = cmd_value__Argv
  Assign = cmd_value__Assign

class part_value_e(object):
  String = 1
  Array = 2
  ExtGlob = 3

_part_value_str = {
  1: 'part_value.String',
  2: 'part_value.Array',
  3: 'part_value.ExtGlob',
}

def part_value_str(tag):
  # type: (int) -> str
  return _part_value_str[tag]

class part_value_t(pybase.CompoundObj):
  def tag_(self):
    # type: () -> int
    return self._type_tag
  pass

class part_value__String(part_value_t):
  _type_tag = 1
  __slots__ = ('s', 'quoted', 'do_split')

  def __init__(self, s, quoted, do_split):
    # type: (str, bool, bool) -> None
    self.s = s
    self.quoted = quoted
    self.do_split = do_split

  @staticmethod
  def Create():
    # type: () -> part_value__String
    return part_value__String('', False, False)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('part_value.String')
    L = out_node.fields

    x0 = NewLeaf(self.s, color_e.StringConst)
    L.append(field('s', x0))

    x1 = hnode.Leaf('T' if self.quoted else 'F', color_e.OtherConst)
    L.append(field('quoted', x1))

    x2 = hnode.Leaf('T' if self.do_split else 'F', color_e.OtherConst)
    L.append(field('do_split', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('part_value.String')
    L = out_node.fields
    x0 = NewLeaf(self.s, color_e.StringConst)
    L.append(field('s', x0))

    x1 = hnode.Leaf('T' if self.quoted else 'F', color_e.OtherConst)
    L.append(field('quoted', x1))

    x2 = hnode.Leaf('T' if self.do_split else 'F', color_e.OtherConst)
    L.append(field('do_split', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class part_value__Array(part_value_t):
  _type_tag = 2
  __slots__ = ('strs',)

  def __init__(self, strs):
    # type: (Optional[List[str]]) -> None
    self.strs = strs

  @staticmethod
  def Create():
    # type: () -> part_value__Array
    return part_value__Array([])

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('part_value.Array')
    L = out_node.fields

    if self.strs is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.strs:
        x0.children.append(NewLeaf(i0, color_e.StringConst))
      L.append(field('strs', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('part_value.Array')
    L = out_node.fields
    if self.strs is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.strs:
        x0.children.append(NewLeaf(i0, color_e.StringConst))
      L.append(field('strs', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class part_value__ExtGlob(part_value_t):
  _type_tag = 3
  __slots__ = ('part_vals',)

  def __init__(self, part_vals):
    # type: (Optional[List[part_value_t]]) -> None
    self.part_vals = part_vals

  @staticmethod
  def Create():
    # type: () -> part_value__ExtGlob
    return part_value__ExtGlob([])

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('part_value.ExtGlob')
    L = out_node.fields

    if self.part_vals is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.part_vals:
        x0.children.append(i0.PrettyTree())
      L.append(field('part_vals', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('part_value.ExtGlob')
    L = out_node.fields
    if self.part_vals is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.part_vals:
        x0.children.append(i0.AbbreviatedTree())
      L.append(field('part_vals', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class part_value(object):
  String = part_value__String
  Array = part_value__Array
  ExtGlob = part_value__ExtGlob

class value_e(object):
  Undef = 1
  Str = 2
  Int = 3
  MaybeStrArray = 4
  AssocArray = 5
  Bool = 6
  Float = 7
  Eggex = 8
  Block = 9
  Obj = 10

_value_str = {
  1: 'value.Undef',
  2: 'value.Str',
  3: 'value.Int',
  4: 'value.MaybeStrArray',
  5: 'value.AssocArray',
  6: 'value.Bool',
  7: 'value.Float',
  8: 'value.Eggex',
  9: 'value.Block',
  10: 'value.Obj',
}

def value_str(tag):
  # type: (int) -> str
  return _value_str[tag]

class value_t(pybase.CompoundObj):
  def tag_(self):
    # type: () -> int
    return self._type_tag
  pass

class value__Undef(value_t):
  _type_tag = 1
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Undef')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Undef')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__Str(value_t):
  _type_tag = 2
  __slots__ = ('s',)

  def __init__(self, s):
    # type: (str) -> None
    self.s = s

  @staticmethod
  def Create():
    # type: () -> value__Str
    return value__Str('')

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Str')
    L = out_node.fields

    x0 = NewLeaf(self.s, color_e.StringConst)
    L.append(field('s', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Str')
    L = out_node.fields
    x0 = NewLeaf(self.s, color_e.StringConst)
    L.append(field('s', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__Int(value_t):
  _type_tag = 3
  __slots__ = ('i',)

  def __init__(self, i):
    # type: (int) -> None
    self.i = i

  @staticmethod
  def Create():
    # type: () -> value__Int
    return value__Int(-1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Int')
    L = out_node.fields

    x0 = hnode.Leaf(str(self.i), color_e.OtherConst)
    L.append(field('i', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Int')
    L = out_node.fields
    x0 = hnode.Leaf(str(self.i), color_e.OtherConst)
    L.append(field('i', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__MaybeStrArray(value_t):
  _type_tag = 4
  __slots__ = ('strs',)

  def __init__(self, strs):
    # type: (Optional[List[str]]) -> None
    self.strs = strs

  @staticmethod
  def Create():
    # type: () -> value__MaybeStrArray
    return value__MaybeStrArray([])

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.MaybeStrArray')
    L = out_node.fields

    if self.strs is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.strs:
        x0.children.append(NewLeaf(i0, color_e.StringConst))
      L.append(field('strs', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.MaybeStrArray')
    L = out_node.fields
    if self.strs is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.strs:
        x0.children.append(NewLeaf(i0, color_e.StringConst))
      L.append(field('strs', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__AssocArray(value_t):
  _type_tag = 5
  __slots__ = ('d',)

  def __init__(self, d):
    # type: (Optional[Dict[str, str]]) -> None
    self.d = d

  @staticmethod
  def Create():
    # type: () -> value__AssocArray
    return value__AssocArray(None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.AssocArray')
    L = out_node.fields

    if self.d is not None:  # map
      m = hnode.Leaf("map", color_e.OtherConst)
      x0 = hnode.Array([m])
      for k0, v0 in self.d.iteritems():
        x0.children.append(NewLeaf(k0, color_e.StringConst))
        x0.children.append(NewLeaf(v0, color_e.StringConst))
      L.append(field('d', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.AssocArray')
    L = out_node.fields
    if self.d is not None:  # map
      m = hnode.Leaf("map", color_e.OtherConst)
      x0 = hnode.Array([m])
      for k0, v0 in self.d.iteritems():
        x0.children.append(NewLeaf(k0, color_e.StringConst))
        x0.children.append(NewLeaf(v0, color_e.StringConst))
      L.append(field('d', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__Bool(value_t):
  _type_tag = 6
  __slots__ = ('b',)

  def __init__(self, b):
    # type: (bool) -> None
    self.b = b

  @staticmethod
  def Create():
    # type: () -> value__Bool
    return value__Bool(False)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Bool')
    L = out_node.fields

    x0 = hnode.Leaf('T' if self.b else 'F', color_e.OtherConst)
    L.append(field('b', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Bool')
    L = out_node.fields
    x0 = hnode.Leaf('T' if self.b else 'F', color_e.OtherConst)
    L.append(field('b', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__Float(value_t):
  _type_tag = 7
  __slots__ = ('f',)

  def __init__(self, f):
    # type: (float) -> None
    self.f = f

  @staticmethod
  def Create():
    # type: () -> value__Float
    return value__Float(0.0)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Float')
    L = out_node.fields

    x0 = hnode.Leaf(str(self.f), color_e.OtherConst)
    L.append(field('f', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Float')
    L = out_node.fields
    x0 = hnode.Leaf(str(self.f), color_e.OtherConst)
    L.append(field('f', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__Eggex(value_t):
  _type_tag = 8
  __slots__ = ('expr', 'as_ere')

  def __init__(self, expr, as_ere):
    # type: (Optional[re_t], str) -> None
    self.expr = expr
    self.as_ere = as_ere

  @staticmethod
  def Create():
    # type: () -> value__Eggex
    return value__Eggex(None, '')

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Eggex')
    L = out_node.fields

    assert self.expr is not None
    x0 = self.expr.PrettyTree()
    L.append(field('expr', x0))

    x1 = NewLeaf(self.as_ere, color_e.StringConst)
    L.append(field('as_ere', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Eggex')
    L = out_node.fields
    assert self.expr is not None
    x0 = self.expr.AbbreviatedTree()
    L.append(field('expr', x0))

    x1 = NewLeaf(self.as_ere, color_e.StringConst)
    L.append(field('as_ere', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__Block(value_t):
  _type_tag = 9
  __slots__ = ('body',)

  def __init__(self, body):
    # type: (Optional[command_t]) -> None
    self.body = body

  @staticmethod
  def Create():
    # type: () -> value__Block
    return value__Block(None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Block')
    L = out_node.fields

    assert self.body is not None
    x0 = self.body.PrettyTree()
    L.append(field('body', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Block')
    L = out_node.fields
    assert self.body is not None
    x0 = self.body.AbbreviatedTree()
    L.append(field('body', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value__Obj(value_t):
  _type_tag = 10
  __slots__ = ('obj',)

  def __init__(self, obj):
    # type: (Any) -> None
    self.obj = obj

  @staticmethod
  def Create():
    # type: () -> value__Obj
    return value__Obj(None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Obj')
    L = out_node.fields

    x0 = hnode.External(self.obj)
    L.append(field('obj', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('value.Obj')
    L = out_node.fields
    x0 = hnode.External(self.obj)
    L.append(field('obj', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class value(object):
  Undef = value__Undef
  Str = value__Str
  Int = value__Int
  MaybeStrArray = value__MaybeStrArray
  AssocArray = value__AssocArray
  Bool = value__Bool
  Float = value__Float
  Eggex = value__Eggex
  Block = value__Block
  Obj = value__Obj

class a_index_e(object):
  Str = 1
  Int = 2

_a_index_str = {
  1: 'a_index.Str',
  2: 'a_index.Int',
}

def a_index_str(tag):
  # type: (int) -> str
  return _a_index_str[tag]

class a_index_t(pybase.CompoundObj):
  def tag_(self):
    # type: () -> int
    return self._type_tag
  pass

class a_index__Str(a_index_t):
  _type_tag = 1
  __slots__ = ('s',)

  def __init__(self, s):
    # type: (str) -> None
    self.s = s

  @staticmethod
  def Create():
    # type: () -> a_index__Str
    return a_index__Str('')

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('a_index.Str')
    L = out_node.fields

    x0 = NewLeaf(self.s, color_e.StringConst)
    L.append(field('s', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('a_index.Str')
    L = out_node.fields
    x0 = NewLeaf(self.s, color_e.StringConst)
    L.append(field('s', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class a_index__Int(a_index_t):
  _type_tag = 2
  __slots__ = ('i',)

  def __init__(self, i):
    # type: (int) -> None
    self.i = i

  @staticmethod
  def Create():
    # type: () -> a_index__Int
    return a_index__Int(-1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('a_index.Int')
    L = out_node.fields

    x0 = hnode.Leaf(str(self.i), color_e.OtherConst)
    L.append(field('i', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('a_index.Int')
    L = out_node.fields
    x0 = hnode.Leaf(str(self.i), color_e.OtherConst)
    L.append(field('i', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class a_index(object):
  Str = a_index__Str
  Int = a_index__Int

class scope_t(pybase.SimpleObj):
  pass

class scope_e(object):
  Parent = scope_t(1)
  Shopt = scope_t(2)
  Dynamic = scope_t(3)
  LocalOrGlobal = scope_t(4)
  LocalOnly = scope_t(5)
  GlobalOnly = scope_t(6)

_scope_str = {
  1: 'scope.Parent',
  2: 'scope.Shopt',
  3: 'scope.Dynamic',
  4: 'scope.LocalOrGlobal',
  5: 'scope.LocalOnly',
  6: 'scope.GlobalOnly',
}

def scope_str(val):
  # type: (scope_t) -> str
  return _scope_str[val]

class lvalue_e(object):
  Named = 1
  Indexed = 2
  Keyed = 3
  ObjIndex = 4
  ObjAttr = 5

_lvalue_str = {
  1: 'lvalue.Named',
  2: 'lvalue.Indexed',
  3: 'lvalue.Keyed',
  4: 'lvalue.ObjIndex',
  5: 'lvalue.ObjAttr',
}

def lvalue_str(tag):
  # type: (int) -> str
  return _lvalue_str[tag]

class lvalue_t(pybase.CompoundObj):
  def tag_(self):
    # type: () -> int
    return self._type_tag
  pass

class lvalue__Named(lvalue_t):
  _type_tag = 1
  __slots__ = ('name', 'blame_spid')

  def __init__(self, name, blame_spid):
    # type: (str, int) -> None
    self.name = name
    self.blame_spid = blame_spid

  @staticmethod
  def Create():
    # type: () -> lvalue__Named
    return lvalue__Named('', -1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.Named')
    L = out_node.fields

    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    x1 = hnode.Leaf(str(self.blame_spid), color_e.OtherConst)
    L.append(field('blame_spid', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.Named')
    L = out_node.fields
    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    x1 = hnode.Leaf(str(self.blame_spid), color_e.OtherConst)
    L.append(field('blame_spid', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class lvalue__Indexed(lvalue_t):
  _type_tag = 2
  __slots__ = ('name', 'index', 'blame_spid')

  def __init__(self, name, index, blame_spid):
    # type: (str, int, int) -> None
    self.name = name
    self.index = index
    self.blame_spid = blame_spid

  @staticmethod
  def Create():
    # type: () -> lvalue__Indexed
    return lvalue__Indexed('', -1, -1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.Indexed')
    L = out_node.fields

    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    x1 = hnode.Leaf(str(self.index), color_e.OtherConst)
    L.append(field('index', x1))

    x2 = hnode.Leaf(str(self.blame_spid), color_e.OtherConst)
    L.append(field('blame_spid', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.Indexed')
    L = out_node.fields
    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    x1 = hnode.Leaf(str(self.index), color_e.OtherConst)
    L.append(field('index', x1))

    x2 = hnode.Leaf(str(self.blame_spid), color_e.OtherConst)
    L.append(field('blame_spid', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class lvalue__Keyed(lvalue_t):
  _type_tag = 3
  __slots__ = ('name', 'key', 'blame_spid')

  def __init__(self, name, key, blame_spid):
    # type: (str, str, int) -> None
    self.name = name
    self.key = key
    self.blame_spid = blame_spid

  @staticmethod
  def Create():
    # type: () -> lvalue__Keyed
    return lvalue__Keyed('', '', -1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.Keyed')
    L = out_node.fields

    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    x1 = NewLeaf(self.key, color_e.StringConst)
    L.append(field('key', x1))

    x2 = hnode.Leaf(str(self.blame_spid), color_e.OtherConst)
    L.append(field('blame_spid', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.Keyed')
    L = out_node.fields
    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    x1 = NewLeaf(self.key, color_e.StringConst)
    L.append(field('key', x1))

    x2 = hnode.Leaf(str(self.blame_spid), color_e.OtherConst)
    L.append(field('blame_spid', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class lvalue__ObjIndex(lvalue_t):
  _type_tag = 4
  __slots__ = ('obj', 'index')

  def __init__(self, obj, index):
    # type: (Any, Any) -> None
    self.obj = obj
    self.index = index

  @staticmethod
  def Create():
    # type: () -> lvalue__ObjIndex
    return lvalue__ObjIndex(None, None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.ObjIndex')
    L = out_node.fields

    x0 = hnode.External(self.obj)
    L.append(field('obj', x0))

    x1 = hnode.External(self.index)
    L.append(field('index', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.ObjIndex')
    L = out_node.fields
    x0 = hnode.External(self.obj)
    L.append(field('obj', x0))

    x1 = hnode.External(self.index)
    L.append(field('index', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class lvalue__ObjAttr(lvalue_t):
  _type_tag = 5
  __slots__ = ('obj', 'attr')

  def __init__(self, obj, attr):
    # type: (Any, str) -> None
    self.obj = obj
    self.attr = attr

  @staticmethod
  def Create():
    # type: () -> lvalue__ObjAttr
    return lvalue__ObjAttr(None, '')

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.ObjAttr')
    L = out_node.fields

    x0 = hnode.External(self.obj)
    L.append(field('obj', x0))

    x1 = NewLeaf(self.attr, color_e.StringConst)
    L.append(field('attr', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('lvalue.ObjAttr')
    L = out_node.fields
    x0 = hnode.External(self.obj)
    L.append(field('obj', x0))

    x1 = NewLeaf(self.attr, color_e.StringConst)
    L.append(field('attr', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class lvalue(object):
  Named = lvalue__Named
  Indexed = lvalue__Indexed
  Keyed = lvalue__Keyed
  ObjIndex = lvalue__ObjIndex
  ObjAttr = lvalue__ObjAttr

class redirect_arg_e(object):
  Path = 1
  CopyFd = 2
  MoveFd = 3
  CloseFd = 4
  HereDoc = 5

_redirect_arg_str = {
  1: 'redirect_arg.Path',
  2: 'redirect_arg.CopyFd',
  3: 'redirect_arg.MoveFd',
  4: 'redirect_arg.CloseFd',
  5: 'redirect_arg.HereDoc',
}

def redirect_arg_str(tag):
  # type: (int) -> str
  return _redirect_arg_str[tag]

class redirect_arg_t(pybase.CompoundObj):
  def tag_(self):
    # type: () -> int
    return self._type_tag
  pass

class redirect_arg__Path(redirect_arg_t):
  _type_tag = 1
  __slots__ = ('filename',)

  def __init__(self, filename):
    # type: (str) -> None
    self.filename = filename

  @staticmethod
  def Create():
    # type: () -> redirect_arg__Path
    return redirect_arg__Path('')

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.Path')
    L = out_node.fields

    x0 = NewLeaf(self.filename, color_e.StringConst)
    L.append(field('filename', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.Path')
    L = out_node.fields
    x0 = NewLeaf(self.filename, color_e.StringConst)
    L.append(field('filename', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class redirect_arg__CopyFd(redirect_arg_t):
  _type_tag = 2
  __slots__ = ('target_fd',)

  def __init__(self, target_fd):
    # type: (int) -> None
    self.target_fd = target_fd

  @staticmethod
  def Create():
    # type: () -> redirect_arg__CopyFd
    return redirect_arg__CopyFd(-1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.CopyFd')
    L = out_node.fields

    x0 = hnode.Leaf(str(self.target_fd), color_e.OtherConst)
    L.append(field('target_fd', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.CopyFd')
    L = out_node.fields
    x0 = hnode.Leaf(str(self.target_fd), color_e.OtherConst)
    L.append(field('target_fd', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class redirect_arg__MoveFd(redirect_arg_t):
  _type_tag = 3
  __slots__ = ('target_fd',)

  def __init__(self, target_fd):
    # type: (int) -> None
    self.target_fd = target_fd

  @staticmethod
  def Create():
    # type: () -> redirect_arg__MoveFd
    return redirect_arg__MoveFd(-1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.MoveFd')
    L = out_node.fields

    x0 = hnode.Leaf(str(self.target_fd), color_e.OtherConst)
    L.append(field('target_fd', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.MoveFd')
    L = out_node.fields
    x0 = hnode.Leaf(str(self.target_fd), color_e.OtherConst)
    L.append(field('target_fd', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class redirect_arg__CloseFd(redirect_arg_t):
  _type_tag = 4
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.CloseFd')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.CloseFd')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class redirect_arg__HereDoc(redirect_arg_t):
  _type_tag = 5
  __slots__ = ('body',)

  def __init__(self, body):
    # type: (str) -> None
    self.body = body

  @staticmethod
  def Create():
    # type: () -> redirect_arg__HereDoc
    return redirect_arg__HereDoc('')

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.HereDoc')
    L = out_node.fields

    x0 = NewLeaf(self.body, color_e.StringConst)
    L.append(field('body', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect_arg.HereDoc')
    L = out_node.fields
    x0 = NewLeaf(self.body, color_e.StringConst)
    L.append(field('body', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class redirect_arg(object):
  Path = redirect_arg__Path
  CopyFd = redirect_arg__CopyFd
  MoveFd = redirect_arg__MoveFd
  CloseFd = redirect_arg__CloseFd
  HereDoc = redirect_arg__HereDoc

class wait_status_e(object):
  Proc = 1
  Pipeline = 2
  Cancelled = 3

_wait_status_str = {
  1: 'wait_status.Proc',
  2: 'wait_status.Pipeline',
  3: 'wait_status.Cancelled',
}

def wait_status_str(tag):
  # type: (int) -> str
  return _wait_status_str[tag]

class wait_status_t(pybase.CompoundObj):
  def tag_(self):
    # type: () -> int
    return self._type_tag
  pass

class wait_status__Proc(wait_status_t):
  _type_tag = 1
  __slots__ = ('code',)

  def __init__(self, code):
    # type: (int) -> None
    self.code = code

  @staticmethod
  def Create():
    # type: () -> wait_status__Proc
    return wait_status__Proc(-1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('wait_status.Proc')
    L = out_node.fields

    x0 = hnode.Leaf(str(self.code), color_e.OtherConst)
    L.append(field('code', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('wait_status.Proc')
    L = out_node.fields
    x0 = hnode.Leaf(str(self.code), color_e.OtherConst)
    L.append(field('code', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class wait_status__Pipeline(wait_status_t):
  _type_tag = 2
  __slots__ = ('codes',)

  def __init__(self, codes):
    # type: (Optional[List[int]]) -> None
    self.codes = codes

  @staticmethod
  def Create():
    # type: () -> wait_status__Pipeline
    return wait_status__Pipeline([])

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('wait_status.Pipeline')
    L = out_node.fields

    if self.codes is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.codes:
        x0.children.append(hnode.Leaf(str(i0), color_e.OtherConst))
      L.append(field('codes', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('wait_status.Pipeline')
    L = out_node.fields
    if self.codes is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.codes:
        x0.children.append(hnode.Leaf(str(i0), color_e.OtherConst))
      L.append(field('codes', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class wait_status__Cancelled(wait_status_t):
  _type_tag = 3
  __slots__ = ('sig_num',)

  def __init__(self, sig_num):
    # type: (int) -> None
    self.sig_num = sig_num

  @staticmethod
  def Create():
    # type: () -> wait_status__Cancelled
    return wait_status__Cancelled(-1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('wait_status.Cancelled')
    L = out_node.fields

    x0 = hnode.Leaf(str(self.sig_num), color_e.OtherConst)
    L.append(field('sig_num', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('wait_status.Cancelled')
    L = out_node.fields
    x0 = hnode.Leaf(str(self.sig_num), color_e.OtherConst)
    L.append(field('sig_num', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class wait_status(object):
  Proc = wait_status__Proc
  Pipeline = wait_status__Pipeline
  Cancelled = wait_status__Cancelled

class flow_t(pybase.SimpleObj):
  pass

class flow_e(object):
  Nothing = flow_t(1)
  Break = flow_t(2)
  Raise = flow_t(3)

_flow_str = {
  1: 'flow.Nothing',
  2: 'flow.Break',
  3: 'flow.Raise',
}

def flow_str(val):
  # type: (flow_t) -> str
  return _flow_str[val]

class span_t(pybase.SimpleObj):
  pass

class span_e(object):
  Black = span_t(1)
  Delim = span_t(2)
  Backslash = span_t(3)

_span_str = {
  1: 'span.Black',
  2: 'span.Delim',
  3: 'span.Backslash',
}

def span_str(val):
  # type: (span_t) -> str
  return _span_str[val]

emit_t = int  # type alias for integer

class emit_i(object):
  Part = 1
  Delim = 2
  Empty = 3
  Escape = 4
  Nothing = 5
  ARRAY_SIZE = 6

_emit_str = {
  1: 'emit.Part',
  2: 'emit.Delim',
  3: 'emit.Empty',
  4: 'emit.Escape',
  5: 'emit.Nothing',
}

def emit_str(val):
  # type: (emit_t) -> str
  return _emit_str[val]

state_t = int  # type alias for integer

class state_i(object):
  Invalid = 1
  Start = 2
  DE_White1 = 3
  DE_Gray = 4
  DE_White2 = 5
  Black = 6
  Backslash = 7
  Done = 8
  ARRAY_SIZE = 9

_state_str = {
  1: 'state.Invalid',
  2: 'state.Start',
  3: 'state.DE_White1',
  4: 'state.DE_Gray',
  5: 'state.DE_White2',
  6: 'state.Black',
  7: 'state.Backslash',
  8: 'state.Done',
}

def state_str(val):
  # type: (state_t) -> str
  return _state_str[val]

char_kind_t = int  # type alias for integer

class char_kind_i(object):
  DE_White = 1
  DE_Gray = 2
  Black = 3
  Backslash = 4
  Sentinel = 5
  ARRAY_SIZE = 6

_char_kind_str = {
  1: 'char_kind.DE_White',
  2: 'char_kind.DE_Gray',
  3: 'char_kind.Black',
  4: 'char_kind.Backslash',
  5: 'char_kind.Sentinel',
}

def char_kind_str(val):
  # type: (char_kind_t) -> str
  return _char_kind_str[val]

class job_state_t(pybase.SimpleObj):
  pass

class job_state_e(object):
  Running = job_state_t(1)
  Done = job_state_t(2)
  Stopped = job_state_t(3)

_job_state_str = {
  1: 'job_state.Running',
  2: 'job_state.Done',
  3: 'job_state.Stopped',
}

def job_state_str(val):
  # type: (job_state_t) -> str
  return _job_state_str[val]

class flag_type_t(pybase.SimpleObj):
  pass

class flag_type_e(object):
  Bool = flag_type_t(1)
  Int = flag_type_t(2)
  Float = flag_type_t(3)
  Str = flag_type_t(4)

_flag_type_str = {
  1: 'flag_type.Bool',
  2: 'flag_type.Int',
  3: 'flag_type.Float',
  4: 'flag_type.Str',
}

def flag_type_str(val):
  # type: (flag_type_t) -> str
  return _flag_type_str[val]

class trace_e(object):
  External = 1
  CommandSub = 2
  ForkWait = 3
  Fork = 4
  PipelinePart = 5
  ProcessSub = 6
  HereDoc = 7

_trace_str = {
  1: 'trace.External',
  2: 'trace.CommandSub',
  3: 'trace.ForkWait',
  4: 'trace.Fork',
  5: 'trace.PipelinePart',
  6: 'trace.ProcessSub',
  7: 'trace.HereDoc',
}

def trace_str(tag):
  # type: (int) -> str
  return _trace_str[tag]

class trace_t(pybase.CompoundObj):
  def tag_(self):
    # type: () -> int
    return self._type_tag
  pass

class trace__External(trace_t):
  _type_tag = 1
  __slots__ = ('argv',)

  def __init__(self, argv):
    # type: (Optional[List[str]]) -> None
    self.argv = argv

  @staticmethod
  def Create():
    # type: () -> trace__External
    return trace__External([])

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.External')
    L = out_node.fields

    if self.argv is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.argv:
        x0.children.append(NewLeaf(i0, color_e.StringConst))
      L.append(field('argv', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.External')
    L = out_node.fields
    if self.argv is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.argv:
        x0.children.append(NewLeaf(i0, color_e.StringConst))
      L.append(field('argv', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class trace__CommandSub(trace_t):
  _type_tag = 2
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.CommandSub')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.CommandSub')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class trace__ForkWait(trace_t):
  _type_tag = 3
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.ForkWait')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.ForkWait')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class trace__Fork(trace_t):
  _type_tag = 4
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.Fork')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.Fork')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class trace__PipelinePart(trace_t):
  _type_tag = 5
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.PipelinePart')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.PipelinePart')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class trace__ProcessSub(trace_t):
  _type_tag = 6
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.ProcessSub')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.ProcessSub')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class trace__HereDoc(trace_t):
  _type_tag = 7
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.HereDoc')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('trace.HereDoc')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class trace(object):
  External = trace__External
  CommandSub = trace__CommandSub
  ForkWait = trace__ForkWait
  Fork = trace__Fork
  PipelinePart = trace__PipelinePart
  ProcessSub = trace__ProcessSub
  HereDoc = trace__HereDoc

class word_style_t(pybase.SimpleObj):
  pass

class word_style_e(object):
  Expr = word_style_t(1)
  Unquoted = word_style_t(2)
  DQ = word_style_t(3)
  SQ = word_style_t(4)

_word_style_str = {
  1: 'word_style.Expr',
  2: 'word_style.Unquoted',
  3: 'word_style.DQ',
  4: 'word_style.SQ',
}

def word_style_str(val):
  # type: (word_style_t) -> str
  return _word_style_str[val]

class assign_arg(pybase.CompoundObj):
  _type_tag = 64
  __slots__ = ('var_name', 'rval', 'plus_eq', 'spid')

  def __init__(self, var_name, rval, plus_eq, spid):
    # type: (str, Optional[Optional[value_t]], bool, int) -> None
    self.var_name = var_name
    self.rval = rval
    self.plus_eq = plus_eq
    self.spid = spid

  @staticmethod
  def Create():
    # type: () -> assign_arg
    return assign_arg('', None, False, -1)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('assign_arg')
    L = out_node.fields

    x0 = NewLeaf(self.var_name, color_e.StringConst)
    L.append(field('var_name', x0))

    if self.rval is not None:  # maybe
      x1 = self.rval.PrettyTree()
      L.append(field('rval', x1))

    x2 = hnode.Leaf('T' if self.plus_eq else 'F', color_e.OtherConst)
    L.append(field('plus_eq', x2))

    x3 = hnode.Leaf(str(self.spid), color_e.OtherConst)
    L.append(field('spid', x3))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('assign_arg')
    L = out_node.fields
    x0 = NewLeaf(self.var_name, color_e.StringConst)
    L.append(field('var_name', x0))

    if self.rval is not None:  # maybe
      x1 = self.rval.AbbreviatedTree()
      L.append(field('rval', x1))

    x2 = hnode.Leaf('T' if self.plus_eq else 'F', color_e.OtherConst)
    L.append(field('plus_eq', x2))

    x3 = hnode.Leaf(str(self.spid), color_e.OtherConst)
    L.append(field('spid', x3))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class VTestPlace(pybase.CompoundObj):
  _type_tag = 65
  __slots__ = ('name', 'index')

  def __init__(self, name, index):
    # type: (str, Optional[a_index_t]) -> None
    self.name = name
    self.index = index

  @staticmethod
  def Create():
    # type: () -> VTestPlace
    return VTestPlace('', None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('VTestPlace')
    L = out_node.fields

    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    assert self.index is not None
    x1 = self.index.PrettyTree()
    L.append(field('index', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('VTestPlace')
    L = out_node.fields
    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    assert self.index is not None
    x1 = self.index.AbbreviatedTree()
    L.append(field('index', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class VarSubState(pybase.CompoundObj):
  _type_tag = 66
  __slots__ = ('join_array', 'is_type_query')

  def __init__(self, join_array, is_type_query):
    # type: (bool, bool) -> None
    self.join_array = join_array
    self.is_type_query = is_type_query

  @staticmethod
  def Create():
    # type: () -> VarSubState
    return VarSubState(False, False)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('VarSubState')
    L = out_node.fields

    x0 = hnode.Leaf('T' if self.join_array else 'F', color_e.OtherConst)
    L.append(field('join_array', x0))

    x1 = hnode.Leaf('T' if self.is_type_query else 'F', color_e.OtherConst)
    L.append(field('is_type_query', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('VarSubState')
    L = out_node.fields
    x0 = hnode.Leaf('T' if self.join_array else 'F', color_e.OtherConst)
    L.append(field('join_array', x0))

    x1 = hnode.Leaf('T' if self.is_type_query else 'F', color_e.OtherConst)
    L.append(field('is_type_query', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class cell(pybase.CompoundObj):
  _type_tag = 67
  __slots__ = ('exported', 'readonly', 'nameref', 'val')

  def __init__(self, exported, readonly, nameref, val):
    # type: (bool, bool, bool, Optional[value_t]) -> None
    self.exported = exported
    self.readonly = readonly
    self.nameref = nameref
    self.val = val

  @staticmethod
  def Create():
    # type: () -> cell
    return cell(False, False, False, None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('cell')
    L = out_node.fields

    x0 = hnode.Leaf('T' if self.exported else 'F', color_e.OtherConst)
    L.append(field('exported', x0))

    x1 = hnode.Leaf('T' if self.readonly else 'F', color_e.OtherConst)
    L.append(field('readonly', x1))

    x2 = hnode.Leaf('T' if self.nameref else 'F', color_e.OtherConst)
    L.append(field('nameref', x2))

    assert self.val is not None
    x3 = self.val.PrettyTree()
    L.append(field('val', x3))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('cell')
    L = out_node.fields
    x0 = hnode.Leaf('T' if self.exported else 'F', color_e.OtherConst)
    L.append(field('exported', x0))

    x1 = hnode.Leaf('T' if self.readonly else 'F', color_e.OtherConst)
    L.append(field('readonly', x1))

    x2 = hnode.Leaf('T' if self.nameref else 'F', color_e.OtherConst)
    L.append(field('nameref', x2))

    assert self.val is not None
    x3 = self.val.AbbreviatedTree()
    L.append(field('val', x3))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class redirect(pybase.CompoundObj):
  _type_tag = 68
  __slots__ = ('op_id', 'op_spid', 'loc', 'arg')

  def __init__(self, op_id, op_spid, loc, arg):
    # type: (Id_t, int, Optional[redir_loc_t], Optional[redirect_arg_t]) -> None
    self.op_id = op_id
    self.op_spid = op_spid
    self.loc = loc
    self.arg = arg

  @staticmethod
  def Create():
    # type: () -> redirect
    return redirect(-1, -1, None, None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect')
    L = out_node.fields

    x0 = hnode.Leaf(Id_str(self.op_id), color_e.UserType)
    L.append(field('op_id', x0))

    x1 = hnode.Leaf(str(self.op_spid), color_e.OtherConst)
    L.append(field('op_spid', x1))

    assert self.loc is not None
    x2 = self.loc.PrettyTree()
    L.append(field('loc', x2))

    assert self.arg is not None
    x3 = self.arg.PrettyTree()
    L.append(field('arg', x3))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('redirect')
    L = out_node.fields
    x0 = hnode.Leaf(Id_str(self.op_id), color_e.UserType)
    L.append(field('op_id', x0))

    x1 = hnode.Leaf(str(self.op_spid), color_e.OtherConst)
    L.append(field('op_spid', x1))

    assert self.loc is not None
    x2 = self.loc.AbbreviatedTree()
    L.append(field('loc', x2))

    assert self.arg is not None
    x3 = self.arg.AbbreviatedTree()
    L.append(field('arg', x3))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class Proc(pybase.CompoundObj):
  _type_tag = 69
  __slots__ = ('name', 'name_spid', 'sig', 'body', 'defaults', 'dynamic_scope')

  def __init__(self, name, name_spid, sig, body, defaults, dynamic_scope):
    # type: (str, int, Optional[proc_sig_t], Optional[command_t], Optional[List[value_t]], bool) -> None
    self.name = name
    self.name_spid = name_spid
    self.sig = sig
    self.body = body
    self.defaults = defaults
    self.dynamic_scope = dynamic_scope

  @staticmethod
  def Create():
    # type: () -> Proc
    return Proc('', -1, None, None, [], False)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('Proc')
    L = out_node.fields

    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    x1 = hnode.Leaf(str(self.name_spid), color_e.OtherConst)
    L.append(field('name_spid', x1))

    assert self.sig is not None
    x2 = self.sig.PrettyTree()
    L.append(field('sig', x2))

    assert self.body is not None
    x3 = self.body.PrettyTree()
    L.append(field('body', x3))

    if self.defaults is not None:  # array
      x4 = hnode.Array([])
      for i4 in self.defaults:
        x4.children.append(i4.PrettyTree())
      L.append(field('defaults', x4))

    x5 = hnode.Leaf('T' if self.dynamic_scope else 'F', color_e.OtherConst)
    L.append(field('dynamic_scope', x5))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('Proc')
    L = out_node.fields
    x0 = NewLeaf(self.name, color_e.StringConst)
    L.append(field('name', x0))

    x1 = hnode.Leaf(str(self.name_spid), color_e.OtherConst)
    L.append(field('name_spid', x1))

    assert self.sig is not None
    x2 = self.sig.AbbreviatedTree()
    L.append(field('sig', x2))

    assert self.body is not None
    x3 = self.body.AbbreviatedTree()
    L.append(field('body', x3))

    if self.defaults is not None:  # array
      x4 = hnode.Array([])
      for i4 in self.defaults:
        x4.children.append(i4.AbbreviatedTree())
      L.append(field('defaults', x4))

    x5 = hnode.Leaf('T' if self.dynamic_scope else 'F', color_e.OtherConst)
    L.append(field('dynamic_scope', x5))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class StatusArray(pybase.CompoundObj):
  _type_tag = 70
  __slots__ = ('codes', 'spids')

  def __init__(self, codes, spids):
    # type: (Optional[List[int]], Optional[List[int]]) -> None
    self.codes = codes
    self.spids = spids

  @staticmethod
  def Create():
    # type: () -> StatusArray
    return StatusArray([], [])

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('StatusArray')
    L = out_node.fields

    if self.codes is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.codes:
        x0.children.append(hnode.Leaf(str(i0), color_e.OtherConst))
      L.append(field('codes', x0))

    if self.spids is not None:  # array
      x1 = hnode.Array([])
      for i1 in self.spids:
        x1.children.append(hnode.Leaf(str(i1), color_e.OtherConst))
      L.append(field('spids', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('StatusArray')
    L = out_node.fields
    if self.codes is not None:  # array
      x0 = hnode.Array([])
      for i0 in self.codes:
        x0.children.append(hnode.Leaf(str(i0), color_e.OtherConst))
      L.append(field('codes', x0))

    if self.spids is not None:  # array
      x1 = hnode.Array([])
      for i1 in self.spids:
        x1.children.append(hnode.Leaf(str(i1), color_e.OtherConst))
      L.append(field('spids', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class CommandStatus(pybase.CompoundObj):
  _type_tag = 71
  __slots__ = ('check_errexit', 'pipe_negated', 'pipe_status', 'pipe_spids',
               'show_code')

  def __init__(self, check_errexit, pipe_negated, pipe_status, pipe_spids,
               show_code):
    # type: (bool, bool, Optional[List[int]], Optional[List[int]], bool) -> None
    self.check_errexit = check_errexit
    self.pipe_negated = pipe_negated
    self.pipe_status = pipe_status
    self.pipe_spids = pipe_spids
    self.show_code = show_code

  @staticmethod
  def Create():
    # type: () -> CommandStatus
    return CommandStatus(False, False, [], [], False)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('CommandStatus')
    L = out_node.fields

    x0 = hnode.Leaf('T' if self.check_errexit else 'F', color_e.OtherConst)
    L.append(field('check_errexit', x0))

    x1 = hnode.Leaf('T' if self.pipe_negated else 'F', color_e.OtherConst)
    L.append(field('pipe_negated', x1))

    if self.pipe_status is not None:  # array
      x2 = hnode.Array([])
      for i2 in self.pipe_status:
        x2.children.append(hnode.Leaf(str(i2), color_e.OtherConst))
      L.append(field('pipe_status', x2))

    if self.pipe_spids is not None:  # array
      x3 = hnode.Array([])
      for i3 in self.pipe_spids:
        x3.children.append(hnode.Leaf(str(i3), color_e.OtherConst))
      L.append(field('pipe_spids', x3))

    x4 = hnode.Leaf('T' if self.show_code else 'F', color_e.OtherConst)
    L.append(field('show_code', x4))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('CommandStatus')
    L = out_node.fields
    x0 = hnode.Leaf('T' if self.check_errexit else 'F', color_e.OtherConst)
    L.append(field('check_errexit', x0))

    x1 = hnode.Leaf('T' if self.pipe_negated else 'F', color_e.OtherConst)
    L.append(field('pipe_negated', x1))

    if self.pipe_status is not None:  # array
      x2 = hnode.Array([])
      for i2 in self.pipe_status:
        x2.children.append(hnode.Leaf(str(i2), color_e.OtherConst))
      L.append(field('pipe_status', x2))

    if self.pipe_spids is not None:  # array
      x3 = hnode.Array([])
      for i3 in self.pipe_spids:
        x3.children.append(hnode.Leaf(str(i3), color_e.OtherConst))
      L.append(field('pipe_spids', x3))

    x4 = hnode.Leaf('T' if self.show_code else 'F', color_e.OtherConst)
    L.append(field('show_code', x4))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

class hay_node(pybase.CompoundObj):
  _type_tag = 72
  __slots__ = ('children',)

  def __init__(self, children):
    # type: (Optional[Dict[str, hay_node]]) -> None
    self.children = children

  @staticmethod
  def Create():
    # type: () -> hay_node
    return hay_node(None)

  def PrettyTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('hay_node')
    L = out_node.fields

    if self.children is not None:  # map
      m = hnode.Leaf("map", color_e.OtherConst)
      x0 = hnode.Array([m])
      for k0, v0 in self.children.iteritems():
        x0.children.append(NewLeaf(k0, color_e.StringConst))
        x0.children.append(v0.PrettyTree())
      L.append(field('children', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> hnode_t
    out_node = NewRecord('hay_node')
    L = out_node.fields
    if self.children is not None:  # map
      m = hnode.Leaf("map", color_e.OtherConst)
      x0 = hnode.Array([m])
      for k0, v0 in self.children.iteritems():
        x0.children.append(NewLeaf(k0, color_e.StringConst))
        x0.children.append(v0.AbbreviatedTree())
      L.append(field('children', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> hnode_t
    return self._AbbreviatedTree()

